import {
  MiddlewareConsumer,
  Module,
  NestModule,
  RequestMethod,
} from '@nestjs/common';
import { CSRFMiddleware } from 'src/middleware/csrf.middleware';
import { FlashMiddleware } from 'src/middleware/flash.middleware';
import { UserMiddleware } from 'src/middleware/user.middleware';
import { ValidateCSRFMiddleware } from 'src/middleware/validate-csrf.middleware';
import { AuditModule } from 'src/modules/objects/audit/audit.module';
import { UserTokenModule } from 'src/modules/objects/user-token/user-token.module';
import { UserModule } from 'src/modules/objects/user/user.module';
import { LoginController } from './login.controller';

@Module({
  imports: [UserModule, UserTokenModule, AuditModule],
  controllers: [LoginController],
})
export class LoginModule implements NestModule {
  configure(consumer: MiddlewareConsumer) {
    consumer.apply(CSRFMiddleware, UserMiddleware).forRoutes(LoginController);
    consumer
      .apply(ValidateCSRFMiddleware)
      .forRoutes({ path: 'login*', method: RequestMethod.POST });

    consumer.apply(FlashMiddleware).forRoutes('login*');
  }
}
