import {
  MiddlewareConsumer,
  Module,
  NestModule,
  RequestMethod,
} from '@nestjs/common';
import { CSRFMiddleware } from 'src/middleware/csrf.middleware';
import { FlashMiddleware } from 'src/middleware/flash.middleware';
import { UserMiddleware } from 'src/middleware/user.middleware';
import { ValidateCSRFMiddleware } from 'src/middleware/validate-csrf.middleware';
import { AuditModule } from 'src/modules/objects/audit/audit.module';
import { UserModule } from 'src/modules/objects/user/user.module';
import { RegisterController } from './register.controller';

@Module({
  imports: [UserModule, AuditModule],
  controllers: [RegisterController],
})
export class RegisterModule implements NestModule {
  configure(consumer: MiddlewareConsumer) {
    consumer
      .apply(CSRFMiddleware, UserMiddleware)
      .forRoutes(RegisterController);

    consumer
      .apply(ValidateCSRFMiddleware)
      .forRoutes({ path: 'register*', method: RequestMethod.POST });

    consumer.apply(FlashMiddleware).forRoutes('register*');
  }
}
